/***************************************************************************
 *   Copyright (C) 2004 by Nicolas Boichat                                 *
 *   nicolas@boichat.ch                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <stdlib.h>
#include <ctype.h>

#include "ddccontrol.h"

void printinfo(ddc_control* dc) {
	printf("Manufacturer: %s\n", dc->getManufacturer());
	printf("Model: %x\n", dc->getModel());
	printf("Serial: %d\n", dc->getSerial());
	printf("Date: Year %d Week %d\n", dc->getYear(), dc->getWeek());
}

int main(int argc, char** argv) {
	int bus = -1;
	int busmax;
	unsigned int idx, nval;
	unsigned char val, max;

	ddc_control* dc;
	
	int state = 0; // 0 - no parameter, 1 - read idx, 2 - write nval to idx, -1 - help

	int probe_only = 0;

	int c;

	printf("DDC/CI control %s, (c) Nicolas Boichat 2004\n", VERSION);

	opterr = 0;
	
	while ((c = getopt (argc, argv, "phb:i:v:")) != -1) {
		switch (c)
		{
			case 'p':
				probe_only = 1;
				break;
			case 'b':
				bus = strtol(optarg, NULL, 0);
				break;
			case 'h':
				state = -1;
			case 'i':
				if (state == 0) state = 1;
				idx = strtol(optarg, NULL, 0);
				break;
			case 'v':
				state = 2;
				nval = strtol(optarg, NULL, 0);
				break;
			case '?':
				if (isprint (optopt))
					fprintf (stderr, "Unknown option `-%c'.\n", optopt);
				else
					fprintf (stderr, "Unknown option character `\\x%x'.\n", optopt);
				return 1;
			default:
				return 2;
		}
	}

	if (state == -1) {
		fprintf(stderr, "Usage :\n %s -i index [-v newvalue]\n", argv[0]);
		return 1;
	}

	if (bus == -1) {
		busmax = 32;
		for (bus = 0; bus < busmax; bus++) {
			printf("Probing bus %d...", bus);
			dc = new ddc_control(bus);
			if (dc->init()) {
				printf("OK.\n");
				printinfo(dc);
				if (probe_only) {
					delete dc;
				}
				else {
					break;
				}
			}
			else {
				if (dc->getError() == DDC_PERMISSION_DENIED) {
					printf("Error, permission denied.\n");
					busmax = 32;
				}
				else if (dc->getError() == DDC_NO_SUCH_FILE_ERROR) {
					printf("None.\n");
					if (busmax-bus > 3) { /* If a file doesn't exists, read only three more */
						busmax = bus + 3;
					}
				}
				else if (dc->getError() == DDC_UNKNOWN_OPEN_ERROR) {
					printf("Error while opening.\n");
					busmax = 32;
				}
				else {
					printf("Probably not a DDC bus (error : %d).\n", dc->getError());
					busmax = 32;
				}
				delete dc;
				dc = NULL;
			}
		}
	}
	else {
		printf("Using bus %d...", bus);
		dc = new ddc_control(bus);
		if (dc->init()) {
			printf("OK.\n");
			printinfo(dc);
			if (probe_only) {
				delete dc;
				dc = NULL;
			}
		}
		else {
			if (dc->getError() == DDC_PERMISSION_DENIED) {
				printf("Error, permission denied.\n");
				busmax = 32;
			}
			else if (dc->getError() == DDC_NO_SUCH_FILE_ERROR) {
				printf("None.\n");
				if (busmax-bus > 3) { /* If a file doesn't exists, read only three more */
					busmax = bus + 3;
				}
			}
			else if (dc->getError() == DDC_UNKNOWN_OPEN_ERROR) {
				printf("Error while opening.\n");
				busmax = 32;
			}
			else {
				printf("Probably not a DDC bus (error : %d).\n", dc->getError());
				busmax = 32;
			}
			dc = NULL;
		}
		if (dc == NULL) {
			return 0;
		}
	}

	if (probe_only)
		return 0;

	if (dc == NULL) {
		fprintf(stderr, "Error, cannot find any DDC/I2C bus.\n");
		fprintf(stderr, "Please check i2c-dev driver and your framebuffer driver (rivafb, radeonfb...) are loaded.\n");
		return 1;
	}

	if (state > 0) {
		if (idx > 0xff) {
			fprintf(stderr, "Error: index > 0xff.\n");
			return 1;
		}
		dc->getValue(idx, &val, &max);
		if (state == 2) {
			printf("Old value : %#x: %x/%x (%d)\n", idx, val, max, val*100/((max != 0) ? max : 1));
			if (nval > max) {
				fprintf(stderr, "Error: new value > maximum (%#x).\n", max);
				return 1;
			}
			dc->setValue(idx, nval);
			dc->getValue(idx, &val, &max);
			printf("New value : %#x: %x/%x (%d%%)\n", idx, val, max, val*100/((max != 0) ? max : 1));
		}
		else {
			printf("Current value : %#x: %x/%x (%d%%)\n", idx, val, max, val*100/((max != 0) ? max : 1));
		}
	}
	
	return 0;
}
